; #NoTrayIcon
Opt("MustDeclareVars", 1)
Opt("TrayIconDebug", 1)

; Program to delete ADS (Alternate Data Streams) from files
; on NTFS partitions
;
; two freeware programs used:
;
; 1) lads.exe from http://www.heysoft.de/nt/ntfs-ads.htm
;    this site also explains a lot about ADS.
; 2) cat.exe as part of a zip file containing many Windows
;    ports of Unix commands at http://unxutils.sourceforge.net/
;
; if not using the compiled version of this script, be sure
; to place the two exes in the same folder as the script file.
;
; If used, stub and stubloader are available through this thread
; on AutoIt's web forums:
; http://www.autoitscript.com/forum/index.php?s=&showtopic=17380&view=findpost&p=119727

Global Const $lads = "lads.exe"
Global Const $cat = "cat.exe"

Global $bConsoleApp = False, $vCmdLine = $CmdLine
Global $bExitSet = False, $sExitMsg = ''

CheckConsole($bConsoleApp, $vCmdLine)
; Temp while compiled and using stubloader
$bConsoleApp = True
Main()
Exit

;**********************************************************************
Func Main()
   Local $i, $sOut, $iColon, $sPath, $sFile, $aOut, $sPrevFile = ''
   Local $Installed[2] = ['', '']

   If Not FileExists(@ScriptDir & '\' & $lads) Then
      If @Compiled Then
         FileInstall("lads.exe", @ScriptDir & '\')
         $Installed[0] = @ScriptDir & '\' & $lads
      Else
         MsgBox(0, @ScriptName, 'Needed file ' & _Quote($lads) & ' not found.')
         Return
      EndIf
   EndIf
   If Not FileExists(@ScriptDir & '\' & $cat) Then
      If @Compiled Then
         FileInstall("cat.exe", @ScriptDir & '\')
         $Installed[1] = @ScriptDir & '\' & $cat
      Else
         MsgBox(0, @ScriptName, 'Needed file ' & _Quote($cat) & ' not found.')
         Return
      EndIf
   EndIf

   ; Print(@ScriptDir)
   ; Print(@WorkingDir)
   If $vCmdLine[0] > 0 Then
      $sPath = _UnQuote($vCmdLine[1])
   Else
      $sPath = @WorkingDir
   EndIf
   Print('Scanning ' & _Quote($sPath))
   $sOut = _RunOut(_Quote(@ScriptDir & '\' & $lads) & ' ' & _Quote($sPath) & ' /s', @WorkingDir, @SW_HIDE)
   $aOut = StringSplit(StringStripCR($sOut), @LF)
   For $i = 11 To UBound($aOut) - 3 ; account for header and footer
      ; Print($aOut[$i])
      $iColon = StringInStr($aOut[$i], ':', 0, -1) ; find last occurance of colon
      If $iColon > 15 Then
         $sFile = StringMid($aOut[$i], 13, $iColon - 13)
         If ($sFile <> $sPrevFile) And FileExists($sFile) Then
            Print($sFile)
            StripFile($sFile)
            $sPrevFile = $sFile
         EndIf
      EndIf
   Next
   For $i = 0 To 1
      If $Installed[$i] <> '' Then
         FileDelete($Installed[$i])
      EndIf
   Next
EndFunc   ;==>Main

Func StripFile($sFile)
   Local $aFileDates[3], $i, $sTempFile, $sAttribs

   For $i = 0 To 2
      $aFileDates[$i] = FileGetTime($sFile, $i, 1)
   Next
   $sAttribs = FileGetAttrib2($sFile)
   FileSetAttrib($sFile, '-R')
   $sTempFile = TempFile($sFile)
   If FileMove($sFile, $sTempFile) Then
      If _FileCopy($sTempFile, $sFile) Then
         For $i = 0 To 2
            FileSetTime($sFile, $aFileDates[$i], $i)
         Next
         FileSetAttrib($sFile, $sAttribs)
         If FileGetAttrib2($sFile) <> $sAttribs Then
            Print("File attributes don't match!")
            Print($sFile & ': ' & FileGetAttrib2($sFile))
            Print('Should be: ' & $sAttribs)
            Exit
         EndIf
         For $i = 0 To 2
            ; If FileGetTime($sFile, $i, 1) <> FileGetTime($sTempFile, $i, 1) Then
            If FileGetTime($sFile, $i, 1) <> $aFileDates[$i] Then
               Print('Timestamp ' & $i & " doesn't match!")
               Print($sFile & ': ' & FileGetTime($sFile, $i, 1))
               ; Print($sTempFile & ': ' & FileGetTime($sTempFile, $i, 1))
               Print('Should be: ' & $aFileDates[$i])
               Exit
            EndIf
         Next
         If Not FileDelete($sTempFile) Then
            Print('Temp file ' & _Quote($sTempFile) & ' could not be deleted.')
            Exit
         EndIf
         FileSetAttrib($sFile, $sAttribs)
      Else
         ; Copying the file failed for some reason. Let's try to rename the file back to what it was
         Print('_FileCopy failed!')
         If Not FileMove($sTempFile, $sFile) Then
            Print('Could not rename temp file ' _Quote($sTempFile) ' back to ' _Quote($sFile))
            Exit
         EndIf
      EndIf
   Else
      Print('Rename failed!: ' & _Quote($sFile) & ' to ' & _Quote($sTempFile) & '. Skipping...')
   EndIf
   If Not FileExists($sFile) Then
      Print(_Quote($sFile) & ' does not exist!!!')
      Exit
   EndIf
EndFunc

Func _FileCopy($sSource, $sDest)
   Local $bResult = False
   _RunDOS($cat & ' ' & _Quote($sSource) & ' >' & _Quote($sDest))
   If FileExists($sDest) Then
      If FileGetSize($sDest) = FileGetSize($sSource) Then
         $bResult = True
      Else
         Print('File size mismatch')
         Exit
      EndIf
   Else
      Print($sDest & ' does not exist')
      Exit
   EndIf
   Return $bResult
EndFunc

Func _FileGetAttrib($sFile)
   Local $sResult = '', $i, $aAttribs[7] = [ 'R', 'A', 'S', 'H', 'N', 'O', 'T']
   Local $sAttribs = FileGetAttrib($sFile)

   For $i = 0 To 6
      If StringInStr($sAttribs, $aAttribs[$i]) > 0 Then
         $sResult &= '+' & $aAttribs[$i]
      Else
         $sResult &= '-' & $aAttribs[$i]
      EndIf
   Next
   Return $sResult
EndFunc

Func FileGetAttrib2($sFile)
   Local $sResult = '', $i, $aAttribs[6] = [ 'R', 'A', 'S', 'H', 'O', 'T']
   Local $sAttribs = FileGetAttrib($sFile)

   For $i = 0 To 5
      If StringInStr($sAttribs, $aAttribs[$i]) > 0 Then
         $sResult &= '+' & $aAttribs[$i]
      Else
         $sResult &= '-' & $aAttribs[$i]
      EndIf
   Next
   Return $sResult
EndFunc

;**********************************************************************
; Generate a name for a temporary file, making sure the file does
; not already exist.
;**********************************************************************
Func TempFile($sFile)
   Local $sTempName
   Do
      $sTempName = "~"
      While StringLen($sTempName) < 4
         $sTempName = $sTempName & Chr(Round(Random(97, 122), 0))
      Wend
      $sTempName = $sFile & "." & $sTempName
   Until Not FileExists($sTempName)
   Return ($sTempName)
EndFunc   ;==>_TempFile

;**********************************************************************
Func Print($sMsg)
   If $bConsoleApp Then
      ConsoleWrite(StringStripWS($sMsg, 3) & @LF)
   Else
      If Not $bExitSet Then
         Opt("OnExitFunc", "PrintOnExit")
      EndIf
      $sExitMsg &= @LF & StringStripWS($sMsg, 3)
      If @TrayIconVisible Then TrayTip(@ScriptName, $sMsg, 1, 1)
   EndIf
EndFunc

Func PrintOnExit()
   If StringLen($sExitMsg) > 1 Then
      MsgBox(0, @ScriptName, StringTrimLeft($sExitMsg, 1), 10)
   EndIf
EndFunc

;**********************************************************************
Func CheckConsole(ByRef $bConsoleApp, ByRef $vCmdLine)
   Local $i
   $bConsoleApp = False
   If $vCmdLine[0] > 0 Then
      If $vCmdLine[1] = '/console' Then
         $bConsoleApp = True
         $vCmdLine[0] -= 1
         For $i = 1 To $vCmdLine[0]
            $vCmdLine[$i] = $vCmdLine[$i+1]
         Next
      EndIf
   EndIf
EndFunc

;**********************************************************************
Func _Quote($sTheString, $sQuoteChar = '"')
   Return ($sQuoteChar & $sTheString & $sQuoteChar)
EndFunc

;**********************************************************************
; Run a "DOS" command without showing any window.
;**********************************************************************
Func _RunDOS($sCommand)
   Return RunWait(@ComSpec & ' /C' & $sCommand, @WorkingDir, @SW_HIDE)
EndFunc   ;==>_RunDOS

;**********************************************************************
; Run a command without showing any window.
; Returns the output of the command.
;**********************************************************************
Func _RunOut($sCommand, $sWorkDir = @WorkingDir, $iShowMode = @SW_HIDE)
   Local $sResult = '', $iPID = Run($sCommand, $sWorkDir, $iShowMode, 2)
   While Not @Error
      $sResult &= StdoutRead($iPID)
   Wend
   Return $sResult
EndFunc;==>_RunOut

;**********************************************************************
Func _UnQuote($sTheString, $sQuoteChar = '"')
   If (StringLeft($sTheString, 1) = $sQuoteChar) And (StringRight($sTheString, 1) = $sQuoteChar) Then
      Return StringMid($sTheString, 2, (StringLen($sTheString) - 2))
   Else
      Return $sTheString
   EndIf
EndFunc
